import dotenv from "dotenv";
import express from "express";
import mongoose from "mongoose";
import cors from "cors";

import templeRouter from "./routes/temple.js";
import usersRouter from "./routes/user.js";
import reviewRouter from "./routes/review.js";

import { createRequire } from 'module';
const require = createRequire(import.meta.url);

dotenv.config();

const app = express();
const port = process.env.PORT || 5000;
const mongooseUri = process.env.ATLAS_URI;
const mongooseOptions = {
  useNewUrlParser: true,
  useCreateIndex: true,
  useUnifiedTopology: true,
  useFindAndModify: true,
};

const swaggerUi = require('swagger-ui-express'), swaggerDocument = require('./swagger.json');

app.use(cors());
app.use(express.json());

mongoose.connect(mongooseUri, mongooseOptions);

const connection = mongoose.connection;

connection.once("open", () => {
  console.log("MongoDB database connection established successfully");
});

app.use("/temples", templeRouter);
app.use("/users", usersRouter);
app.use("/reviews", reviewRouter);

app.use('/api-docs', swaggerUi.serve, swaggerUi.setup(swaggerDocument));

app.listen(port, () => {
  console.log(`Server is running on port: ${port}`);
});

