import React, { useState, useEffect } from "react";
import StarRating from "./StarRating";
import WeatherCard from "./WeatherCard";
import UvCard from "./UvCard";
import ReviewCard from "./ReviewCard";
import axios from "axios";
import TempleMap from "./TempleMap"
import TempleNearby from "./TempleNearby";
import "./css/Card.css";



function Card(props) {
  let userId = props.userId;
  let templeId = props.templeId;
  const [templeInfo, setTempleInfo] = useState({ reviews: [] });

  const AnyReactComponent = ({ text }) => <div>{text}</div>;

  useEffect(() => {
    axios
      .get(`http://localhost:5000/temples/${templeId}/TempleInfo`)
      .then((res) => setTempleInfo(res.data));
  }, []);



  return (
    <>
      <div className="temple-card">
        {/* <img src={temple} alt="temple" /> */}


        <div className="card-a">
          <div className="card-l">
            <div className="temple-name">
              <h1>{templeInfo.temple_name}</h1>
            </div>
            <div className="weather-uv">
              <WeatherCard templeInfo={templeInfo} />
              <UvCard templeInfo={templeInfo} />
            </div>
            <div className="map">
              <TempleMap lat={templeInfo.lat} long={templeInfo.long} templeId={templeId} />

            </div>
          </div>
          <div className="card-r">
            <div className="rating">
              <StarRating rating={templeInfo.avg_rating} size="50" />
              <span className="r">{templeInfo.avg_rating}/5 </span>
              <span>Ratings</span>
            </div>
            <div className="review">
              <ReviewCard reviews={templeInfo.reviews} userId={userId} templeId={templeId} />
            </div>
            <div className="nearby">
              <TempleNearby templeId={templeId} />

            </div>
          </div>
        </div>

      </div>
    </>
  );
}

export default Card;
