import React, { useState } from "react";
import "./css/component.css";
import { FaStar } from "react-icons/fa";
import StarRating from "./StarRating";

function ReviewCard(props) {
  let reviews = props.reviews;
  let userId = props.userId;
  let templeId = props.templeId;
  const [review, setReview] = useState("");
  const [star, setStar] = useState("");
  const [rating, setRating] = useState(null);
  const [hover, setHover] = useState(null);

  const responseReview = (response) => {
    const requestOptions = {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: JSON.stringify({
        userid: String(userId),
        stars: rating,
        title: "Review",
        content: review,
        date: "string",
      }),
    };
    fetch(
      `http://127.0.0.1:5000/reviews/${templeId}`,
      requestOptions
    ).then((response) => response.json());
  };

  return (
    <div className="review-card">
      <h3 className="review-text">Reviews</h3>
      <div className="box">
        {reviews.map((review, i) => (
          <div className="reviews-box" key={i}>
            <p>{review.content}</p>
            <StarRating className="review-rating" rating={review.stars} size="25" />
          </div>
        ))}
      </div>
      <div className="reviews-add">
        <div className="reviews-add">
        { userId !== "" ? 
          (
          <form onSubmit={responseReview}>
            <div>
              <textarea
                className="comment-box"
                placeholder="Tell us, how do you feel about this temple..."
                onChange={(event) => setReview(event.target.value)}
                value={review}
              ></textarea>
            </div>
            <div>
              <div className="set-rating">
                {[...Array(5)].map((star, i) => {
                  const ratingValue = i + 1;

                  return (
                    <label key={i}>
                      <input
                        type="radio"
                        name="rating"
                        value={ratingValue}
                        onClick={() => setRating(ratingValue)}
                      />
                      <FaStar
                        id={i}
                        className="star"
                        color={ratingValue <= (hover || rating) ? "#ffc107" : "#e4e5e9"}
                        size={40}
                        onMouseEnter={() => setHover(ratingValue)}
                        onMouseLeave={() => setHover(null)}
                      />
                    </label>
                  );
                })}
              </div>

            </div>
            <input type="submit" value="Submit" />
          </form>)
          : (<div>Login to review</div>)
        }
        </div>
      </div>
    </div>
  );

}


export default ReviewCard;
