import bs4
import requests
import os
import json
from dotenv import load_dotenv
from pathlib import Path  # Python 3.6+ only
from datetime import datetime
import random
import string
from math import sin, cos, sqrt, atan2, radians

load_dotenv()

GOOGLE_API_KEY = os.getenv('GOOGLE_API_KEY')

#  get temple list
url = "https://th.wikipedia.org/wiki/%E0%B8%A3%E0%B8%B2%E0%B8%A2%E0%B8%8A%E0%B8%B7%E0%B9%88%E0%B8%AD%E0%B8%A7%E0%B8%B1%E0%B8%94%E0%B9%83%E0%B8%99%E0%B8%81%E0%B8%A3%E0%B8%B8%E0%B8%87%E0%B9%80%E0%B8%97%E0%B8%9E%E0%B8%A1%E0%B8%AB%E0%B8%B2%E0%B8%99%E0%B8%84%E0%B8%A3"
r = requests.get(url)
soup = bs4.BeautifulSoup(r.text, 'html.parser')
select_links = soup.find_all('a')
temples = []


def random_string(str_size):
    chars = string.ascii_letters + string.punctuation
    return ''.join(random.choice(chars) for x in range(str_size))


def create_user(name):
    api_url = f'http://127.0.0.1:3000/users'
    gen_email = name.split()[0].lower() + '@gmail.com'
    req_body = {
        "name": name,
        "email": gen_email
    }
    req = requests.post(api_url, json=req_body)
    # check if user already exist
    if req.status_code == 400:
        get_req = requests.get(api_url)
        users = get_req.json()
        for user in users:
            if user['name'] == name:
                return user['_id']
            elif user['name'] != name and user['email'] == gen_email:
                gen_email = gen_email[0:-10] + \
                    random_string(5) + gen_email[-10:]
                req_body = {
                    "name": name,
                    "email": gen_email
                }
                req = requests.post(api_url, json=req_body)
    res = req.json()
    return res['_id']


def get_distance_from_lat_lon(lat1, lon1, lat2, lon2):
    # approximate radius of earth in km
    R = 6371
    dlat = radians(lat2 - lat1)
    dlon = radians(lon2 - lon1)

    a = sin(dlat / 2)**2 + cos(lat1) * cos(lat2) * sin(dlon / 2)**2
    c = 2 * atan2(sqrt(a), sqrt(1 - a))

    distance = R * c
    return round(distance, 3)


for link in select_links:
    link_title = str(link.string)
    if link_title[0:3] == 'วัด' and len(link_title) > 3 and link_title != "วัดในกรุงเทพมหานคร":
        temples.append({"name": link.string})

# get place_id from google
for temple in temples[91:101]:
    google_url = f"https://maps.googleapis.com/maps/api/place/findplacefromtext/json?key={GOOGLE_API_KEY}&input={temple['name']}&inputtype=textquery"
    r = requests.get(google_url)
    try:
        place_id = (r.json()['candidates'][0]['place_id'])
    except:
        print("An exception occurred")
        place_id = 'null'
        temple['place_id'] = 'null'
    temple['place_id'] = place_id

# get lat, long, address, engName, rating
request_field = 'name,rating,geometry/location,formatted_address'
for temple in temples[91:101]:
    google_url = f'https://maps.googleapis.com/maps/api/place/details/json?place_id={temple["place_id"]}&fields={request_field}&key={GOOGLE_API_KEY}'
    r = requests.get(google_url).json()['result']
    temple['address'] = r['formatted_address']
    temple['lat'] = r['geometry']['location']['lat']
    temple['long'] = r['geometry']['location']['lng']
    if 'rating' in r:
        temple['rating'] = r['rating']

# get reviews from google api
request_field = 'reviews'
for temple in temples[91:101]:
    google_url = f'https://maps.googleapis.com/maps/api/place/details/json?place_id={temple["place_id"]}&fields={request_field}&key={GOOGLE_API_KEY}'
    r = requests.get(google_url).json()['result']
    temple['reviews'] = []
    if 'reviews' in r:
        res_reviews = r['reviews']
        for r in res_reviews:
            review = {
                "author_name": r['author_name'],
                "stars": r['rating'],
                "title": ' '.join(str(r['text']).split()[0:2]),
                "content": r['text'],
                "date": r['time']
            }
            temple['reviews'].append(review)

# insert temples and reviews into db
for temple in temples[91:101]:
    api_url = f'http://127.0.0.1:3000/temples'
    req_body = {
        "name": temple['name'],
        "description": "",
        "address": temple['address'],
        "lat": temple['lat'],
        "long": temple['long']
    }
    temple_req = requests.post(api_url, json=req_body)
    temple_res = temple_req.json()
    print("temple created: " + temple_res['name'])
    for review in temple['reviews']:
        if review != []:
            # create user from author_name
            user_id = create_user(review['author_name'])
            # insert review into temple by temple id
            temple_id = temple_res['_id']
            review_api_url = f'http://127.0.0.1:3000/reviews/{temple_id}'
            req_body = {
                "userid": user_id,
                "stars": int(review['stars']),
                "title": review['title'],
                "content": review['content'],
                "date": str(datetime.fromtimestamp(review['date']))
            }
            review_req = requests.post(review_api_url, json=req_body)
            print("review created: ", end="")

# find nearest five temples and insert into database
temples_url = f'http://127.0.0.1:5000/temples/'
res = requests.get(temples_url).json()

nearby_temple = []
for temple in res:
    distances = []
    other_temples = list(res)
    other_temples.remove(temple)
    current_lat = temple["lat"]
    current_lon = temple["long"]

    for t in other_temples:
        t_lat = t['lat']
        t_lon = t['long']
        distance = get_distance_from_lat_lon(current_lat, current_lon, t_lat, t_lon)
        dis = {
            "temple": t['_id'],
            "distance": distance
        }
        distances.append(dis)

    top_five = sorted(distances, key = lambda i: i['distance'])[:5]
    nearby_temple.append({"id": temple['_id'], "nearby": top_five})

for temple in nearby_temple:
    req_body = { 
        "nearby_temple": temple['nearby'] 
    }
    response = requests.put(temples_url + temple['id'], json=req_body)